@@ -24,8 +24,10 @@
 namespace diversityForest {
 
 TreeRegression::TreeRegression(std::vector<std::vector<size_t>>& child_nodeIDs, std::vector<size_t>& split_varIDs,
-    std::vector<double>& split_values) :
-    Tree(child_nodeIDs, split_varIDs, split_values), counter(0), sums(0) {
+    std::vector<double>& split_values, std::vector<size_t>& split_types, 
+	std::vector<std::vector<size_t>>& split_multvarIDs, std::vector<std::vector<std::vector<bool>>>& split_directs, 
+	std::vector<std::vector<std::vector<double>>>& split_multvalues) :
+    Tree(child_nodeIDs, split_varIDs, split_values, split_types, split_multvarIDs, split_directs, split_multvalues), counter(0), sums(0) {
 }
 
 void TreeRegression::allocateMemory() {
@@ -142,6 +144,60 @@ bool TreeRegression::splitNodeUnivariateInternal(size_t nodeID, std::vector<std:
   return false;
 }
 
+// asdf: New function: Split node using univariate, binary splitting:
+bool TreeRegression::splitNodeMultivariateInternal(size_t nodeID, std::vector<size_t> sampled_split_types, std::vector<std::vector<size_t>> sampled_split_multvarIDs, std::vector<std::vector<std::vector<bool>>> sampled_split_directs, std::vector<std::vector<std::vector<double>>> sampled_split_multvalues) {
+	
+	// Stop, if no suitable split was found:
+	if (sampled_split_types.size() == 0)
+    {
+	split_multvalues[nodeID].resize(1);
+	split_multvalues[nodeID][0].resize(1);
+	split_multvalues[nodeID][0][0] = estimate(nodeID);
+    return true;
+    }
+	
+  // Stop if maximum node size or depth reached
+    size_t num_samples_node = end_pos[nodeID] - start_pos[nodeID];
+  if (num_samples_node <= min_node_size || (nodeID >= last_left_nodeID && max_depth > 0 && depth >= max_depth)) {
+	split_multvalues[nodeID].resize(1);
+	split_multvalues[nodeID][0].resize(1);
+	split_multvalues[nodeID][0][0] = estimate(nodeID);
+    return true;
+  }
+  
+  // Check if node is pure and set split_value to estimate and stop if pure
+  bool pure = true;
+  double pure_value = 0;
+  for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+    size_t sampleID = sampleIDs[pos];
+    double value = data->get(sampleID, dependent_varID);
+    if (pos != start_pos[nodeID] && value != pure_value) {
+      pure = false;
+      break;
+    }
+    pure_value = value;
+  }
+  if (pure) {
+	split_multvalues[nodeID].resize(1);
+	split_multvalues[nodeID][0].resize(1);
+    split_multvalues[nodeID][0][0] = pure_value;
+    return true;
+  }  
+
+  // Find best split, stop if no decrease of impurity
+  bool stop = findBestSplitMultivariate(nodeID, sampled_split_types, sampled_split_multvarIDs, sampled_split_directs, sampled_split_multvalues);
+    
+  if (stop) {
+	split_multvalues[nodeID].resize(1);
+	split_multvalues[nodeID][0].resize(1);
+    split_multvalues[nodeID][0][0] = estimate(nodeID);
+    return true;
+  }
+
+  return false;
+}
+
+
 void TreeRegression::createEmptyNodeInternal() {
 // Empty on purpose
 }
@@ -152,7 +208,13 @@ double TreeRegression::computePredictionAccuracyInternal() {
   double sum_of_squares = 0;
   for (size_t i = 0; i < num_predictions; ++i) {
     size_t terminal_nodeID = prediction_terminal_nodeIDs[i];
-    double predicted_value = split_values[terminal_nodeID];
+    double predicted_value;
+	if (divfortype == 1) {
+      predicted_value = split_values[terminal_nodeID];
+	}
+	if (divfortype == 2) {
+	  predicted_value = split_multvalues[terminal_nodeID][0][0];
+	}
     double real_value = data->get(oob_sampleIDs[i], dependent_varID);
     if (predicted_value != real_value) {
       sum_of_squares += (predicted_value - real_value) * (predicted_value - real_value);
@@ -310,7 +372,126 @@ bool TreeRegression::findBestSplitUnivariate(size_t nodeID, std::vector<std::pai
   return false;
   
 }
+
+// asdf: New function: Split node using univariate, binary splitting:
+bool TreeRegression::findBestSplitMultivariate(size_t nodeID, std::vector<size_t> sampled_split_types, std::vector<std::vector<size_t>> sampled_split_multvarIDs, std::vector<std::vector<std::vector<bool>>> sampled_split_directs, std::vector<std::vector<std::vector<double>>> sampled_split_multvalues) {
+ 
+
+  size_t num_samples_node = end_pos[nodeID] - start_pos[nodeID];
+  double best_decrease = -1;
+  size_t best_split_type;
+  std::vector<size_t> best_split_multvarID;
+  std::vector<std::vector<bool>> best_split_direct;
+  std::vector<std::vector<double>> best_split_multvalue;
   
+    // Compute sum of responses in node
+    double sum_node = 0;
+    for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+      size_t sampleID = sampleIDs[pos];
+      sum_node += data->get(sampleID, dependent_varID);
+    }
+    
+  // Cycle through the covariate/split pairs and
+  // determine the best split out of these:
+  /////////////////
+  
+
+  // Cycle through the splits and determine the best split
+  // out of these:
+  
+    for (size_t i = 0; i < sampled_split_types.size(); ++i) {
+
+      // -1 because no split possible at largest value
+      ///const size_t num_splits = possible_split_values.size() - 1;
+      double sums_right = 0;
+      size_t n_right = 0;
+      
+	    for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+    size_t sampleID = sampleIDs[pos];
+    double response = data->get(sampleID, dependent_varID);
+	bool inrectangle = IsInRectangle(data, sampleID, sampled_split_types[i], sampled_split_multvarIDs[i], sampled_split_directs[i], sampled_split_multvalues[i]);
+      if (!inrectangle) {
+        ++n_right;
+        sums_right += response;
+      }
+  }
+      
+      // Number of samples in left child:
+      size_t n_left = num_samples_node - n_right;
+      
+	  // Sums:
+      double sum_right = sums_right;
+      double sum_left = sum_node - sum_right;
+      
+	  // Decrease:
+	  double decrease = sum_left * sum_left / (double) n_left + sum_right * sum_right / (double) n_right;
+      
+	  // Some information I returned to the console while developing:
+      ///Rcpp::Rcout << "varIDtemp: " << varIDtemp << std::endl;
+      ///Rcpp::Rcout << "valuetemp: " << valuetemp << std::endl;
+      ///Rcpp::Rcout << "sum_left: " << sum_left << std::endl;
+      ///Rcpp::Rcout << "n_left: " << n_left << std::endl;
+      ///Rcpp::Rcout << "sum_right: " << sum_right << std::endl;
+      ///Rcpp::Rcout << "n_right: " << n_right << std::endl;
+      ///Rcpp::Rcout << "decrease: " << decrease << std::endl;
+      
+    // If better than before, use this
+    if (decrease > best_decrease) {
+
+  size_t nvars = sampled_split_multvarIDs[i].size();
+  best_split_multvarID.resize(nvars);
+  size_t nrects = sampled_split_directs[i].size();
+  best_split_direct.resize(nrects);
+  best_split_multvalue.resize(nrects);
+
+for (size_t j = 0; j < nrects; j++) {
+best_split_direct[j].resize(nvars);
+best_split_multvalue[j].resize(nvars);
+}
+
+      best_split_type = sampled_split_types[i];
+      best_split_multvarID = sampled_split_multvarIDs[i];
+      best_split_direct = sampled_split_directs[i];
+      best_split_multvalue = sampled_split_multvalues[i];
+      best_decrease = decrease;
+    }
+	
+    }
+	
+  // Stop if no good split found
+  if (best_decrease < 0) {
+    return true;
+  }
+
+  // Save best values samma
+  //// Rcpp::Rcout << "Laenge split_types[nodeID]:  " << split_types.size() << std::endl;
+  //// Rcpp::Rcout << "nodeID:  " << nodeID << std::endl;
+  //// Rcpp::Rcout << "best_split_type:  " << best_split_type << std::endl;
+  
+  split_types[nodeID] = best_split_type;
+  
+  split_multvarIDs[nodeID].resize(best_split_multvarID.size());
+  split_multvarIDs[nodeID] = best_split_multvarID;
+
+
+  size_t sizeouter = best_split_direct.size();
+  
+  split_directs[nodeID].resize(sizeouter);
+  for (size_t i = 0; i < sizeouter; ++i) {
+	split_directs[nodeID][i].resize(best_split_direct[i].size());
+  }
+  split_directs[nodeID] = best_split_direct;
+  
+  split_multvalues[nodeID].resize(sizeouter);
+  for (size_t i = 0; i < sizeouter; ++i) {
+	split_multvalues[nodeID][i].resize(best_split_multvalue[i].size());
+  }
+  split_multvalues[nodeID] = best_split_multvalue;
+  
+  return false;
+ 
+}
+
 void TreeRegression::findBestSplitValueSmallQ(size_t nodeID, size_t varID, double sum_node, size_t num_samples_node,
     double& best_value, size_t& best_varID, double& best_decrease) {
 
