@@ -31,9 +31,11 @@ TreeSurvival::TreeSurvival(std::vector<double>* unique_timepoints, size_t status
 }
 
 TreeSurvival::TreeSurvival(std::vector<std::vector<size_t>>& child_nodeIDs, std::vector<size_t>& split_varIDs,
-    std::vector<double>& split_values, std::vector<std::vector<double>> chf, std::vector<double>* unique_timepoints,
+    std::vector<double>& split_values, std::vector<size_t>& split_types, 
+	std::vector<std::vector<size_t>>& split_multvarIDs, std::vector<std::vector<std::vector<bool>>>& split_directs, 
+	std::vector<std::vector<std::vector<double>>>& split_multvalues, std::vector<std::vector<double>> chf, std::vector<double>* unique_timepoints,
     std::vector<size_t>* response_timepointIDs) :
-    Tree(child_nodeIDs, split_varIDs, split_values), status_varID(0), unique_timepoints(unique_timepoints), response_timepointIDs(
+    Tree(child_nodeIDs, split_varIDs, split_values, split_types, split_multvarIDs, split_directs, split_multvalues), status_varID(0), unique_timepoints(unique_timepoints), response_timepointIDs(
         response_timepointIDs), chf(chf), num_deaths(0), num_samples_at_risk(0) {
   this->num_timepoints = unique_timepoints->size();
 }
@@ -107,6 +109,13 @@ bool TreeSurvival::splitNodeUnivariateInternal(size_t nodeID, std::vector<std::p
   return findBestSplitUnivariate(nodeID, sampled_varIDs_values);
 }
 
+// asdf: New function: Split node using univariate, binary splitting:
+bool TreeSurvival::splitNodeMultivariateInternal(size_t nodeID, std::vector<size_t> sampled_split_types, std::vector<std::vector<size_t>> sampled_split_multvarIDs, std::vector<std::vector<std::vector<bool>>> sampled_split_directs, std::vector<std::vector<std::vector<double>>> sampled_split_multvalues) {
+
+  // Find best split, stop if no improvement in split criterion value
+  return findBestSplitMultivariate(nodeID, sampled_split_types, sampled_split_multvarIDs, sampled_split_directs, sampled_split_multvalues);
+}
+
 bool TreeSurvival::findBestSplit(size_t nodeID, std::vector<size_t>& possible_split_varIDs) {
 
   double best_decrease = -1;
@@ -279,6 +288,156 @@ bool TreeSurvival::findBestSplitUnivariate(size_t nodeID, std::vector<std::pair<
 
 }
 
+// asdf: New function: Split node using univariate, binary splitting:
+bool TreeSurvival::findBestSplitMultivariate(size_t nodeID, std::vector<size_t> sampled_split_types, std::vector<std::vector<size_t>> sampled_split_multvarIDs, std::vector<std::vector<std::vector<bool>>> sampled_split_directs, std::vector<std::vector<std::vector<double>>> sampled_split_multvalues) {
+ 
+  double best_logrank = -1;
+  size_t num_samples_node = end_pos[nodeID] - start_pos[nodeID];
+  size_t best_split_type;
+  std::vector<size_t> best_split_multvarID;
+  std::vector<std::vector<bool>> best_split_direct;
+  std::vector<std::vector<double>> best_split_multvalue;
+  
+  computeDeathCounts(nodeID);
+  
+  // Stop if maximum node size or depth reached (will check again for each child node)
+  if (num_samples_node <= min_node_size || (nodeID >= last_left_nodeID && max_depth > 0 && depth >= max_depth)) {
+    computeSurvival(nodeID);
+    return true;
+  }
+  
+	  // Stop early if no split possible
+  if (num_samples_node >= 2 * min_node_size) {
+
+  // Cycle through the covariate/split pairs and
+  // determine the best split out of these:
+  /////////////////
+  
+
+  // Cycle through the splits and determine the best split
+  // out of these:
+  
+    for (size_t i = 0; i < sampled_split_types.size(); ++i) {
+      
+  // Initialize
+  std::vector<size_t> num_deaths_right_child(num_timepoints);
+  std::vector<size_t> delta_samples_at_risk_right_child(num_timepoints);
+  size_t num_samples_right_child = 0;
+    double numerator = 0;
+    double denominator_squared = 0;
+
+    // Count deaths in right child per timepoint
+    for (size_t pos = start_pos[nodeID]; pos < end_pos[nodeID]; ++pos) {
+      size_t sampleID = sampleIDs[pos];
+      size_t survival_timeID = (*response_timepointIDs)[sampleID];
+      
+	  bool inrectangle = IsInRectangle(data, sampleID, sampled_split_types[i], sampled_split_multvarIDs[i], sampled_split_directs[i], sampled_split_multvalues[i]);
+      if (!inrectangle) {
+      // If in right child, count
+      // In right child, if bitwise splitID at position factorID is 1
+        ++num_samples_right_child;
+        ++delta_samples_at_risk_right_child[survival_timeID];
+        if (data->get(sampleID, status_varID) == 1) {
+          ++num_deaths_right_child[survival_timeID];
+        }
+      }
+
+    }
+
+    // Stop if minimal node size reached
+    size_t num_samples_left_child = num_samples_node - num_samples_right_child;
+    if (num_samples_right_child < min_node_size || num_samples_left_child < min_node_size) {
+      continue;
+    }
+
+    // Compute logrank test statistic for this split
+    size_t num_samples_at_risk_right_child = num_samples_right_child;
+    for (size_t t = 0; t < num_timepoints; ++t) {
+      if (num_samples_at_risk[t] < 2 || num_samples_at_risk_right_child < 1) {
+        break;
+      }
+
+      if (num_deaths[t] > 0) {
+        // Numerator and demoninator for log-rank test, notation from Ishwaran et al.
+        double di = (double) num_deaths[t];
+        double di1 = (double) num_deaths_right_child[t];
+        double Yi = (double) num_samples_at_risk[t];
+        double Yi1 = (double) num_samples_at_risk_right_child;
+        numerator += di1 - Yi1 * (di / Yi);
+        denominator_squared += (Yi1 / Yi) * (1.0 - Yi1 / Yi) * ((Yi - di) / (Yi - 1)) * di;
+      }
+
+      // Reduce number of samples at risk for next timepoint
+      num_samples_at_risk_right_child -= delta_samples_at_risk_right_child[t];
+    }
+    double logrank = -1;
+    if (denominator_squared != 0) {
+      logrank = fabs(numerator / sqrt(denominator_squared));
+    }
+
+	    // If better than before, use this
+    if (logrank > best_logrank) {
+		
+  size_t nvars = sampled_split_multvarIDs[i].size();
+  best_split_multvarID.resize(nvars);
+  size_t nrects = sampled_split_directs[i].size();
+  best_split_direct.resize(nrects);
+  best_split_multvalue.resize(nrects);
+
+for (size_t j = 0; j < nrects; j++) {
+best_split_direct[j].resize(nvars);
+best_split_multvalue[j].resize(nvars);
+}
+
+      best_split_type = sampled_split_types[i];
+      best_split_multvarID = sampled_split_multvarIDs[i];
+      best_split_direct = sampled_split_directs[i];
+      best_split_multvalue = sampled_split_multvalues[i];
+      best_logrank = logrank;
+    }
+	
+  }
+
+}
+  
+  
+    // Stop and save CHF if no good split found (this is terminal node).
+  if (best_logrank < 0) {
+    computeSurvival(nodeID);
+    return true;
+  }
+ 
+  // Save best values samma
+  //// Rcpp::Rcout << "Laenge split_types[nodeID]:  " << split_types.size() << std::endl;
+  //// Rcpp::Rcout << "nodeID:  " << nodeID << std::endl;
+  //// Rcpp::Rcout << "best_split_type:  " << best_split_type << std::endl;
+  
+  split_types[nodeID] = best_split_type;
+  
+  split_multvarIDs[nodeID].resize(best_split_multvarID.size());
+  split_multvarIDs[nodeID] = best_split_multvarID;
+
+
+  size_t sizeouter = best_split_direct.size();
+  
+  split_directs[nodeID].resize(sizeouter);
+  for (size_t i = 0; i < sizeouter; ++i) {
+	split_directs[nodeID][i].resize(best_split_direct[i].size());
+  }
+  split_directs[nodeID] = best_split_direct;
+  
+  split_multvalues[nodeID].resize(sizeouter);
+  for (size_t i = 0; i < sizeouter; ++i) {
+	split_multvalues[nodeID][i].resize(best_split_multvalue[i].size());
+  }
+  split_multvalues[nodeID] = best_split_multvalue;
+  
+  return false;
+ 
+}
+
+
+
 bool TreeSurvival::findBestSplitMaxstat(size_t nodeID, std::vector<size_t>& possible_split_varIDs) {
 
   size_t num_samples_node = end_pos[nodeID] - start_pos[nodeID];
